@@ -28,6 +28,7 @@
 
 #include "goadaemon.h"
 #include "goabackend/goabackend.h"
+#include "goabackend/goaprovider-priv.h"
 #include "goabackend/goautils.h"
 
 struct _GoaDaemon
@@ -88,6 +89,52 @@ static void goa_daemon_reload_configuration (GoaDaemon *self);
 
 G_DEFINE_TYPE (GoaDaemon, goa_daemon, G_TYPE_OBJECT);
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+typedef struct
+{
+  GError **error;
+  GList **out_providers;
+  GMainLoop *loop;
+  gboolean op_res;
+} GetAllSyncData;
+
+static void
+get_all_providers_sync_cb (GObject       *source_object,
+                           GAsyncResult  *res,
+                           gpointer       user_data)
+{
+  GetAllSyncData *data = (GetAllSyncData *) user_data;
+
+  data->op_res = goa_provider_get_all_finish (data->out_providers, res, data->error);
+  g_main_loop_quit (data->loop);
+}
+
+static gboolean
+get_all_providers_sync (GCancellable  *cancellable,
+                        GList        **out_providers,
+                        GError       **error)
+{
+  GetAllSyncData data;
+
+  data.error = error;
+  data.out_providers = out_providers;
+
+  /* HACK: Since telepathy-glib doesn't use the thread-default
+   * GMainContext for invoking the asynchronous callbacks, we can't
+   * push a new GMainContext here.
+   */
+  data.loop = g_main_loop_new (NULL, FALSE);
+
+  goa_provider_get_all (get_all_providers_sync_cb, &data);
+  g_main_loop_run (data.loop);
+  g_main_loop_unref (data.loop);
+
+  return data.op_res;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static void
 goa_daemon_finalize (GObject *object)
 {
@@ -210,30 +257,13 @@ on_network_monitor_network_changed (GoaDaemon *self, gboolean available)
   queue_check_credentials (self);
 }
 
-#ifdef GOA_KERBEROS_ENABLED
-static void
-activate_identity_service (GoaDaemon *self)
-{
-  GoaProvider *provider;
-
-  /* We activate the identity service implicitly by using the kerberos
-   * backend.  This way if the kerberos backend isn't enabled, we don't
-   * end up starting the identity service needlessly
-   */
-  provider = goa_provider_get_for_provider_type (GOA_KERBEROS_NAME);
-
-  if (provider != NULL)
-    {
-      g_debug ("activated kerberos provider");
-      g_object_unref (provider);
-    }
-}
-#endif
-
 static void
 goa_daemon_init (GoaDaemon *self)
 {
   static volatile GQuark goa_error_domain = 0;
+  GError *error;
+  GList *l;
+  GList *providers = NULL;
   GoaObjectSkeleton *object;
   gchar *path;
 
@@ -243,6 +273,22 @@ goa_daemon_init (GoaDaemon *self)
   goa_error_domain = GOA_ERROR;
   goa_error_domain; /* shut up -Wunused-but-set-variable */
 
+  error = NULL;
+  if (!get_all_providers_sync (NULL, &providers, &error))
+    {
+      g_warning ("Unable to get the list of providers: %s (%s, %d)",
+                 error->message,
+                 g_quark_to_string (error->domain),
+                 error->code);
+      g_error_free (error);
+    }
+
+  for (l = providers; l != NULL; l = l->next)
+    {
+      GoaProvider *provider = GOA_PROVIDER (l->data);
+      goa_provider_initialize (provider);
+    }
+
   /* TODO: maybe nicer to pass in a GDBusConnection* construct property */
   self->connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
 
@@ -291,10 +337,6 @@ goa_daemon_init (GoaDaemon *self)
   g_dbus_object_manager_server_set_connection (self->object_manager, self->connection);
 
   queue_check_credentials (self);
-
-#ifdef GOA_KERBEROS_ENABLED
-  activate_identity_service (self);
-#endif
 }
 
 static void
@@ -985,14 +1027,77 @@ on_manager_handle_add_account (GoaManager             *manager,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+typedef struct
+{
+  GoaObject *object;
+  GList *invocations;
+} ObjectInvocationData;
+
+static ObjectInvocationData *
+object_invocation_data_new (GoaObject             *object,
+                 GDBusMethodInvocation *invocation)
+{
+  ObjectInvocationData *data;
+  data = g_slice_new0 (ObjectInvocationData);
+  data->object = g_object_ref (object);
+  data->invocations = g_list_prepend (data->invocations, invocation);
+  return data;
+}
+
+static void
+object_invocation_data_unref (ObjectInvocationData *data)
+{
+  g_list_free (data->invocations);
+  g_object_unref (data->object);
+  g_slice_free (ObjectInvocationData, data);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+remove_account_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  GoaDaemon *self;
+  GDBusMethodInvocation *invocation;
+  GError *error;
+  GoaAccount *account;
+  GoaProvider *provider = GOA_PROVIDER (source_object);
+  ObjectInvocationData *data;
+
+  self = GOA_DAEMON (g_task_get_source_object (task));
+  data = g_task_get_task_data (task);
+
+  error= NULL;
+  if (!goa_provider_remove_account_finish (provider, res, &error))
+    {
+      g_warning ("goa_provider_remove_account() failed: %s (%s, %d)",
+                 error->message,
+                 g_quark_to_string (error->domain),
+                 error->code);
+      g_error_free (error);
+    }
+
+  goa_daemon_reload_configuration (self);
+
+  account = goa_object_peek_account (data->object);
+  invocation = G_DBUS_METHOD_INVOCATION (data->invocations->data);
+  goa_account_complete_remove (account, invocation);
+
+  g_object_unref (task);
+}
+
 static gboolean
 on_account_handle_remove (GoaAccount            *account,
                           GDBusMethodInvocation *invocation,
                           gpointer               user_data)
 {
   GoaDaemon *self = GOA_DAEMON (user_data);
+  GoaObject *object;
   GoaProvider *provider;
   GKeyFile *key_file;
+  GTask *task;
+  ObjectInvocationData *data;
   const gchar *provider_type;
   gchar *path;
   gchar *group;
@@ -1003,6 +1108,7 @@ on_account_handle_remove (GoaAccount            *account,
   path = NULL;
   group = NULL;
   key_file = NULL;
+  task = NULL;
 
   if (goa_account_get_is_locked (account))
     {
@@ -1081,12 +1187,17 @@ on_account_handle_remove (GoaAccount            *account,
       goto out;
     }
 
-  goa_daemon_reload_configuration (self);
+  object = GOA_OBJECT (g_dbus_interface_get_object (G_DBUS_INTERFACE (account)));
+  data = object_invocation_data_new (object, invocation);
 
-  goa_account_complete_remove (account, invocation);
+  task = g_task_new (self, NULL, NULL, NULL);
+  g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
+
+  goa_provider_remove_account (provider, object, NULL, remove_account_cb, g_object_ref (task));
 
  out:
   g_clear_object (&provider);
+  g_clear_object (&task);
   g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_free);
   g_free (group);
   g_free (path);
@@ -1095,31 +1206,6 @@ on_account_handle_remove (GoaAccount            *account,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-typedef struct
-{
-  GoaObject *object;
-  GList *invocations;
-} EnsureData;
-
-static EnsureData *
-ensure_data_new (GoaObject             *object,
-                 GDBusMethodInvocation *invocation)
-{
-  EnsureData *data;
-  data = g_slice_new0 (EnsureData);
-  data->object = g_object_ref (object);
-  data->invocations = g_list_prepend (data->invocations, invocation);
-  return data;
-}
-
-static void
-ensure_data_unref (EnsureData *data)
-{
-  g_list_free (data->invocations);
-  g_object_unref (data->object);
-  g_slice_free (EnsureData, data);
-}
-
 static gboolean
 is_authorization_error (GError *error)
 {
@@ -1173,10 +1259,10 @@ ensure_credentials_queue_collector (GObject *source_object, GAsyncResult *res, g
 {
   GTask *task = G_TASK (user_data);
   GoaDaemon *self;
-  EnsureData *data;
   GoaAccount *account;
   GoaProvider *provider = GOA_PROVIDER (source_object);
   GError *error;
+  ObjectInvocationData *data;
   gint expires_in;
 
   self = GOA_DAEMON (g_task_get_source_object (task));
@@ -1240,10 +1326,10 @@ ensure_credentials_queue_sort (gconstpointer a, gconstpointer b, gpointer user_d
 static void
 ensure_credentials_queue_check (GoaDaemon *self)
 {
-  EnsureData *data;
   GoaAccount *account;
   GoaProvider *provider = NULL;
   GTask *task;
+  ObjectInvocationData *data;
   const gchar *id;
   const gchar *provider_type;
   gint64 timestamp;
@@ -1259,7 +1345,7 @@ ensure_credentials_queue_check (GoaDaemon *self)
   task = G_TASK (g_queue_peek_head (self->ensure_credentials_queue));
   self->ensure_credentials_running = TRUE;
 
-  data = (EnsureData *) g_task_get_task_data (task);
+  data = (ObjectInvocationData *) g_task_get_task_data (task);
   account = goa_object_peek_account (data->object);
 
   id = goa_account_get_id (account);
@@ -1298,7 +1384,7 @@ ensure_credentials_queue_coalesce (GoaDaemon *self, GoaObject *object, GDBusMeth
     {
       GoaAccount *account_queued;
       GTask *task = G_TASK (l->data);
-      EnsureData *data;
+      ObjectInvocationData *data;
       const gchar *id_queued;
 
       data = g_task_get_task_data (task);
@@ -1328,10 +1414,10 @@ on_account_handle_ensure_credentials (GoaAccount            *account,
                                       gpointer               user_data)
 {
   GoaDaemon *self = GOA_DAEMON (user_data);
-  EnsureData *data;
   GoaObject *object;
   GoaProvider *provider = NULL;
   GTask *task = NULL;
+  ObjectInvocationData *data;
   const gchar *id;
   const gchar *method_name;
   const gchar *provider_type;
@@ -1367,11 +1453,11 @@ on_account_handle_ensure_credentials (GoaAccount            *account,
       goto out;
     }
 
-  data = ensure_data_new (object, invocation);
+  data = object_invocation_data_new (object, invocation);
 
   task = g_task_new (self, NULL, NULL, NULL);
   g_task_set_priority (task, G_PRIORITY_HIGH);
-  g_task_set_task_data (task, data, (GDestroyNotify) ensure_data_unref);
+  g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
   g_queue_push_tail (self->ensure_credentials_queue, g_object_ref (task));
 
   ensure_credentials_queue_check (self);
@@ -1399,11 +1485,11 @@ goa_daemon_check_credentials (GoaDaemon *self)
   objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (self->object_manager));
   for (l = objects; l != NULL; l = l->next)
     {
-      EnsureData *data;
       GoaAccount *account;
       GoaObject *object = GOA_OBJECT (l->data);
       GoaProvider *provider = NULL;
       GTask *task = NULL;
+      ObjectInvocationData *data;
       const gchar *id;
       const gchar *provider_type;
       gint64 timestamp;
@@ -1435,11 +1521,11 @@ goa_daemon_check_credentials (GoaDaemon *self)
           continue;
         }
 
-      data = ensure_data_new (object, NULL);
+      data = object_invocation_data_new (object, NULL);
 
       task = g_task_new (self, NULL, NULL, NULL);
       g_task_set_priority (task, G_PRIORITY_LOW);
-      g_task_set_task_data (task, data, (GDestroyNotify) ensure_data_unref);
+      g_task_set_task_data (task, data, (GDestroyNotify) object_invocation_data_unref);
       g_queue_push_tail (self->ensure_credentials_queue, g_object_ref (task));
 
       g_clear_object (&provider);
